%% RUN CONTOUR
% writes ContourSolve.txt, ContourSolve.xls with contour parameters, saves
% ContourSolve.mat, structure with all the contour variables
clear all;clc;


%% INPUT PARAMETERS
% picture name
NameInput = 'data.jpg';

% pixel size
PixelSize = 1e-2;

% lover area limit for contours to be included into the database
DownlimitArea = 2;

% compute distances of neighbours: 0 - no, 1 - yes
ComputeDistances = 1;

% method to compute distances: 0 - exact, 1 - approximate
DistanceMethod = 1;

% use parallel mode for computing distances: 0 - serial, 1 - parallel
ParallelMode = 0;

% maximal distance for neighbours
MaxDist = 5;

% number of curve segments for searching minimal distances between curves
% (uses partition of intervals). Recommended several values to try if
% nonregular curves.
nParts = 200;


%% RECOGNIZING CONTOURS FROM THE PICTURE
% loading data
fprintf('=== RECOGNIZING PATTERN ===\n');
fprintf('Reading data...\n');
tic;
Image = imread(NameInput);
BB = size(Image,2); % width of the picture in pixels
HH = size(Image,1); % height of the picture in pixels
% conversion to bw image
fprintf('Converting to black & white...\n');
Image = im2bw(Image,0.5); % constant determinig level between b and w, default 0.5
fprintf('Contour recognition...\n');
% enlarging image due to boundary contours
EnlargedImage = ones(HH+2,BB+2);
EnlargedImage(2:end-1,2:end-1) = Image;
figure(156);clf;hold all;
imshow(EnlargedImage,'InitialMagnification','fit');axis('equal','xy');
title('Original image converted to bw and recognized contours');xlabel('X (1-BB)');ylabel('Y (1-HH)');
[C,handle] = imcontour(EnlargedImage,1,'r');


%% COMPUTING CONTOUR PARAMETERS
figure(157);clf;hold all;axis('equal','xy',[1*PixelSize,BB*PixelSize,1*PixelSize,HH*PixelSize]);
title('Contours used for computation');xlabel('X (1-BB)'),ylabel('Y (1-HH)');
SumArea = 0;
j = 1; % contour ID
k = 1; % accepted contour ID
m = 1; % position measure in C matrix
while m < length(C)
    cLength = C(2,m);
    StepSize = C(1,m)*PixelSize;
    X = C(1,m+1:m+cLength)*PixelSize;
    Y = C(2,m+1:m+cLength)*PixelSize;
    % shifted coordinates - accuracy
    TX = mean(X,2);
    TY = mean(Y,2);
    Xs = X-TX;
    Ys = Y-TY;
    % area
    Area = 1/2*sum((Xs(1:end-1)-Xs(2:end)).*(Y(1:end-1)+Y(2:end)));
    if Area > DownlimitArea
        % remaining parameters
        Length = sum(sqrt((Xs(2:end)-Xs(1:end-1)).^2+(Ys(2:end)-Ys(1:end-1)).^2));
        Sxx = 1/6*sum((Xs(1:end-1)-Xs(2:end)).*(Ys(1:end-1).^2+Ys(1:end-1).*Ys(2:end)+Ys(2:end).^2));
        Syy = 1/6*sum((Ys(2:end)-Ys(1:end-1)).*(Xs(1:end-1).^2+Xs(1:end-1).*Xs(2:end)+Xs(2:end).^2));
        Ixx = 1/12*sum((Xs(1:end-1)-Xs(2:end)).*(Ys(1:end-1).^3+Ys(1:end-1).^2.*Ys(2:end)+Ys(1:end-1).*Ys(2:end).^2+Ys(2:end).^3));
        Iyy = 1/12*sum((Ys(2:end)-Ys(1:end-1)).*(Xs(1:end-1).^3+Xs(1:end-1).^2.*Xs(2:end)+Xs(1:end-1).*Xs(2:end).^2+Xs(2:end).^3));
        Ixy = 1/24*sum((Xs(1:end-1)-Xs(2:end)).*(3*(Xs(1:end-1).*Ys(1:end-1).^2+Xs(2:end).*Ys(2:end).^2)...
            +2*Ys(1:end-1).*Ys(2:end).*(Xs(1:end-1)+Xs(2:end))+Xs(1:end-1).*Ys(2:end).^2+Xs(2:end).*Ys(1:end-1).^2));
        % local centroid in shifted coordinates
        CentroidLocalX = Syy/Area;
        CentroidLocalY = Sxx/Area;
        % global centroid in original coordinates
        CentroidX = Syy/Area+TX;
        CentroidY = Sxx/Area+TY;
        Iuu = Ixx-Area*CentroidLocalY^2;
        Ivv = Iyy-Area*CentroidLocalX^2;
        Iuv = Ixy-Area*CentroidLocalX*CentroidLocalY;
        J = Iuu+Ivv;
        Ixx = Iuu+Area*CentroidY^2;
        Iyy = Ivv+Area*CentroidX^2;
        Ixy = Iuv+Area*CentroidX*CentroidY;
        I = [Iuu -Iuv
            -Iuv Ivv];
        [EigVec,EigVal] = eig(I);
        I1 = EigVal(1,1);
        I2 = EigVal(2,2);
        Angle1 = atan2(EigVec(2,1),EigVec(1,1));
        Angle2 = atan2(EigVec(2,2),EigVec(1,2));
        % print some of the contour parameters
        fprintf('\n%d. contour\nTx = %g, Ty = %g\nA = %g, L = %g\nI1 = %g, I2 = %g, alpha1 = %g rad\n',k,CentroidX,...
            CentroidY,Area,Length,I1,I2,Angle1);
        % database results entry
        Database(k).XY = [X;Y];
        Database(k).Data = [k,CentroidX,CentroidY,Area,Length,Iuu,Ivv,Iuv,I1,Angle1,I2,Angle2,J];
        SumArea = SumArea+Area;
        % draw accepted contour
        plot(CentroidX,CentroidY,'X','color',[1,0,0]);
        text(CentroidX,CentroidY,num2str(k),'hor','center','ver','top');
        plot(X,Y,'b');
        k = k+1;
    end
    % shift to the next contour
    m = m+cLength+1;
    j = j+1;
end
% fraction of inclusion to matrix in the picture with inhomogenities
FractionWith = bwarea(~Image)/(BB*HH);
fprintf('\nFraction of black, with inhomogeneity: %.4g\n',FractionWith);
% fraction of inclusion to matrix in the picture without inhomogenities
FractionWithout = SumArea/(BB*HH*PixelSize^2);
fprintf('Fraction of black, without inhomogeneity: %.4g\n',FractionWithout);
% contour summary
fprintf('\nContour summary\n%d contours totally\n%d contours skipped\n%d contours used\n'...
    ,j-1,j-k,k-1);


%% CONTOUR DISTANCES AND NEIGHBOURING CURVES
if ComputeDistances == 1
    fprintf('\nComputing distances...\n');
    % compute minimal distance
    N = nchoosek(length(Database),2);
    Distance = zeros(N,3);
    Distance(:,1:2) = nchoosek(1:length(Database),2);
    if ParallelMode == 0
        for i = 1:N
            Radius(i) = minimalDistance(Database(Distance(i,1)).XY,Database(Distance(i,2)).XY,MaxDist,nParts,DistanceMethod);
        end
    elseif ParallelMode == 1
        % open matlabpool
        distcomp.feature('LocalUseMpiexec',false);
        if matlabpool('size') == 0
            matlabpool local;
        end
        parfor i = 1:N
            Radius(i) = minimalDistance(Database(Distance(i,1)).XY,Database(Distance(i,2)).XY,MaxDist,nParts,DistanceMethod);
        end
    end
    Distance(:,3) = Radius;
    % find neighbours F < MaxDist
    for i = 1:length(Database)
        Items1 = find(Distance(:,1)==i);
        Items2 = find(Distance(:,2)==i);
        % prehozeni prvych dvou sloupku v i2-tych radcich
        dist_swap = Distance;
        zal = dist_swap(Items2,1);
        dist_swap(Items2,1) = dist_swap(Items2,2);
        dist_swap(Items2,2) = zal;
        % pole potencialnich sousedu a vybrani tech s r < maxdist
        pot_neighbours = dist_swap([Items1;Items2],:);
        j = find(pot_neighbours(:,3)<MaxDist);
        Neighbours = pot_neighbours(j,:);
        % vzestupne serazeni sousednich krivek
        [~,n] = sort(Neighbours(:,2));
        Neighbours = Neighbours(n,:);
        % prirazeni sousednich krivek do struktury
        Database(i).Neighbours = Neighbours;
        % vypis
        fprintf('\nNeighbours of contour %d\n',i);
        for k = 1:size(Neighbours,1)
            fprintf('%d min. dist. = %g\n',Neighbours(k,2),Neighbours(k,3));
        end
    end
end


%% SAVE OUTPUTS
% save database to database.m
save('ContourSolve','Database','BB','HH','PixelSize');
% geometric properties
Y = zeros(length(Database),13);
for i = 1:length(Database)
    Y(i,1:13) = Database(i).Data;
end
% neighbours
if ComputeDistances == 1
    Z = zeros(length(Database),3);
    k = 1;
    for i = 1:length(Database)
        write = Database(i).Neighbours;
        m = size(write,1);
        Z(k:k+m-1,:) = write;
        k = k+m;
    end
end
% write to output.txt file
fprintf('\nWriting to output files...\n');
fid = fopen('ContourSolve.txt','w');
fprintf(fid,'\nVOLUME FRACTIONS\n----------------\n\n');
fprintf(fid,'Fr. with % .4e\nFr. without % .4e\n',FractionWith,FractionWithout);
fprintf(fid,'\n\nGEOMETRIC DATA\n--------------\n\n');
fprintf(fid,'ID Tx Ty A L Iuu Ivv Iuv I1 alpha1 I2 alpha2 J\n');
fprintf(fid,'%4d  % .4e % .4e % .4e % .4e % .4e % .4e % .4e % .4e % .4e % .4e % .4e % .4e\n',Y');
if ComputeDistances == 1
    fprintf(fid,'\n\nNEIGHBOURING CURVES\n-------------------\n\n');
    fprintf(fid,'%4d  %4d % .4e\n',Z');
end
status = fclose('all');
% write to output.xls
xlswrite('ContouSolve.xls',{'GEOMETRIC DATA'},'Hodnoty','B1');
xlswrite('ContouSolve.xls',{'Fr. with', FractionWith;'Fr. without', FractionWithout},'Hodnoty','B2');
xlswrite('ContouSolve.xls',{'ID','Tx','Ty','A','L','Iuu','Ivv','Iuv','I1','alpha1','I2','alpha2','J'},'Hodnoty','B5');
xlswrite('ContouSolve.xls',Y,'Hodnoty','B6');
if ComputeDistances == 1
    xlswrite('ContouSolve.xls',{'NEIGHBOURING CURVES'},'Hodnoty','P1');
    xlswrite('ContouSolve.xls',{'Parent','Neighb.','Min dist.'},'Hodnoty','P2');
    xlswrite('ContouSolve.xls',Z,'Hodnoty','P3');
end
fprintf('Done!\nTime consumed %.1f sec\n',toc);
