%% PDE SOLUTION USING MATLAB
% displacement vector u = [ux;uy]
% defStr field defStr = [[u1x,u2y,u3z,u1y+u2x], [sigX,sigY,sigZ,tauXY]]
% mesh data point matrix p, edge matrix e, triangle matrix t

function [u,defStr,p,e,t] = matlabSolve(PlaneMode,EModulus,Nu,Thickness,NodeSupportX,NodeSupportY,NodeDisplcContrlX,NodeDisplcContrlY,NodeForceX,NodeForceY,Fx,Fy,ux,uy)

% MATLAB ASSEMBLY AND RUN
clc;
fprintf('=== MATLAB SOLUTION ===\n');
fprintf('assembling PDE...\n');
tic;
load('mesh','p','e','t');
% stifness matrix
GModulus = EModulus/(2*(1+Nu));
if PlaneMode == 0 % plane stress
    mu = 2*GModulus*Nu/(1-Nu);
elseif PlaneMode == 1 % plane strain
    mu = 2*GModulus*Nu/(1-2*Nu);
end
c = [2*GModulus+mu;0;GModulus;0;GModulus;mu;0;GModulus;0;2*GModulus+mu];
[K] = assema(p,t,c,[0 0;0 0],[0;0])*Thickness;
nUnknown = size(K,1);
nNode = size(K,1)/2;
fprintf('solving...\n');
u = zeros(nUnknown,1);
f = zeros(nUnknown,1);
% Dirichlet BC
u([NodeSupportX,NodeSupportY+nNode,NodeDisplcContrlX,...
    NodeDisplcContrlY+nNode]) = [zeros(length(NodeSupportX),1)...
    ;zeros(length(NodeSupportY),1);ux*ones(length(NodeDisplcContrlX),1);...
    uy*ones(length(NodeDisplcContrlY),1)];
% nodal forces
f([NodeForceX,NodeForceY+nNode]) = [Fx*ones(length(NodeForceX),1);...
    Fy*ones(length(NodeForceY),1)];
% solution system of equations
unknowns = setdiff(1:nUnknown,[NodeSupportX,NodeSupportY+nNode,...
    NodeDisplcContrlX,NodeDisplcContrlY+nNode]);
reactions = union(NodeSupportX,[NodeSupportY+nNode,NodeDisplcContrlX,...
    NodeDisplcContrlY+nNode]);
u(unknowns) = K(unknowns,unknowns)\(f(unknowns)-K(unknowns,reactions)*u(reactions));
f(reactions) = K(reactions,unknowns)*u(unknowns)+K(reactions,reactions)*u(reactions);
% check for linear momentum balance
if abs(sum(f)) > 1e-5
    error('Balance of linear momentum is not accurate!');
end
% displacement gradient
[u1x,u1y] = pdegrad(p,t,u(1:nNode));
[u2x,u2y] = pdegrad(p,t,u(nNode+1:end));
% material stiffness matrix
D = [2*GModulus+mu mu 0
    mu 2*GModulus+mu 0
    0 0 GModulus];
sig = D*[u1x;u2y;u1y+u2x];
% defStr field: [[u1x,u2y,u3z,u1y+u2x], [sigX,sigY,sigZ,tauXY]]
if PlaneMode == 0
    EpsZ = -Nu/EModulus*(sig(1,:)+sig(2,:));
    defStr = [u1x',u2y',EpsZ',u1y'+u2x',sig(1:2,:)',zeros(length(t),1),sig(3,:)'];
elseif PlaneMode == 1
    SigZ = Nu*(sig(1,:)+sig(2,:));
    defStr = [u1x',u2y',zeros(length(t),1),u1y'+u2x',sig(1:2,:)',SigZ',sig(3,:)'];
end
% write output file
SupportNodes = union(NodeSupportX,[NodeSupportY,NodeDisplcContrlX,...
    NodeDisplcContrlY]);
fid = fopen('PdeMatlabSolve.txt','w');
fprintf(fid,'\nDISPLACEMENTS OUTPUT:\n---------------------\n\n');
for i = 1:nUnknown/2
    fprintf(fid,'Node %6u: ux = % .4e, uy = % .4e\n',i,u(i),u(i+nNode));
end
fprintf(fid,'\n\n\nELEMENT OUTPUT:\n---------------\n');
for i = 1:length(t)
    fprintf(fid,'Element %6u: EpsX = % .4e, EpsY = % .4e, EpsZ = % .4e, GamXY = % .4e\n\t\tSigX = % .4e, SigY = % .4e, SigZ = % .4e, TauXY = % .4e\n',...
        i,defStr(i,1),defStr(i,2),defStr(i,3),defStr(i,4),defStr(i,5),defStr(i,6),defStr(i,7),defStr(i,8));
end
fprintf(fid,'\n\n\nREACTIONS OUTPUT:\n-----------------\n');
for i = 1:length(SupportNodes)
    fprintf(fid,'Node %6u: Rx = % .4e, Ry = % .4e\n',SupportNodes(i),...
        f(SupportNodes(i)),f(SupportNodes(i)+nNode));
end
fclose('all');
fprintf('Time consumed %.1f sec\n\n',toc);

end
